from __future__ import print_function
import sys


def log_to_error(msg, source=None, target_log=sys.stderr):
    # pass
    print(msg + ". Source: " + (source if source is not None else "Not specified"), file=target_log)



from shexer.model.IRI import IRI

def create_IRI_from_string(an_str):
    return IRI(content=an_str)


def create_IRIs_from_string_list(str_list):
    result = []
    for an_str in str_list:
        result.append(create_IRI_from_string(an_str))
    return result




from shexer.shaper import Shaper

target_classes = ["<https://schema.org/DataRecord>",
                  "<https://schema.org/DefinedTerm>",
                  "<https://schema.org/PropertyValue>",
                  "<https://schema.org/Protein>",
                  "<https://schema.org/SequenceAnnotation>",
                  "<https://schema.org/SequenceRange>",
                  "<https://schema.org/ScholarlyArticle>",
                  "<https://schema.org/Organization>",
                  "<https://schema.org/Dataset>",
                  "<https://schema.org/Person>",
                  "<https://schema.org/DefinedTermSet>",
                  "<https://schema.org/DataCatalog>",
                  "<https://schema.org/CreativeWork>",
                  "<https://schema.org/Taxon>",
                  "<https://schema.org/AnatomicalStructure>",
                  "<https://schema.org/ChemicalSubstance>",
                  "<https://schema.org/MedicalCondition>",
                  "<https://schema.org/Gene>",
                  "<https://schema.org/Drug>",
                  "<https://schema.org/MedicalSignOrSymptom>"]

namespaces = {"http://schema.org/": "schema",
              "http://www.wikidata.org/prop/direct/": "wdt",
              "http://www.wikidata.org/prop/": "p",
              "http://www.w3.org/1999/02/22-rdf-syntax-ns#": "rdf",
              "https://schema.org/": "schema",
              "http://www.w3.org/2001/XMLSchema#": "xsd"}

shaper = Shaper(url_endpoint="http://156.35.94.155:8889/bigdata/sparql",
                depth_for_building_subgraph=1,
                track_classes_for_entities_at_last_depth_level=False,
                namespaces_dict=namespaces,
                target_classes=target_classes,
                disable_comments=False,
                limit_remote_instances=30,
                disable_exact_cardinality=True)
result = shaper.shex_graph(string_output=True)
print(result)
print("Done!")



STARTING_CHAR_FOR_SHAPE_NAME = "@"


class Shape(object):

    def __init__(self, name, class_uri, statements):
        self._name = name
        self._class_uri = class_uri
        self._statements = statements

    @property
    def name(self):
        return self._name

    @property
    def class_uri(self):
        return self._class_uri

    @property
    def n_statements(self):
        return len(self._statements)

    @property
    def statements(self):
        return self._statements

    @statements.setter
    def statements(self, statements):
        self._statements = statements

    def yield_statements(self):
        for a_statement in self._statements:
            yield a_statement

    def sort_statements(self, callback, reverse=False):
        self._statements.sort(key=lambda x :callback(x), reverse=reverse)



import requests

################ url_web_service

url_web_service = "http://156.35.86.6:8080/shexer"
# url_web_service = "http://localhost/shexer"



################ namespace

# This ones are inclided by default:

# default_namespaces = {"http://www.w3.org/1999/02/22-rdf-syntax-ns#": "rdf",
#                       "http://www.w3.org/2000/01/rdf-schema#": "rdfs",
#                       "http://www.w3.org/2001/XMLSchema#": "xml",
#                       "http://www.w3.org/XML/1998/namespace/": "xml",
#                       "http://www.w3.org/2002/07/owl#": "owl",
#
#                       }

new_namespaces = {
    "http://wikiba.se/ontology#": "wikibase",
    "http://www.bigdata.com/rdf#": "bd",
    "http://www.wikidata.org/entity/": "wd",
    "http://www.wikidata.org/prop/direct/": "wdt",
    "http://www.wikidata.org/prop/direct-normalized/": "wdtn",
    "http://www.wikidata.org/entity/statement/": "wds",
    "http://www.wikidata.org/prop/": "p",
    "http://www.wikidata.org/reference/": "wdref",
    "http://www.wikidata.org/value/": "wdv",
    "http://www.wikidata.org/prop/statement/": "ps",
    "http://www.wikidata.org/prop/statement/value/": "psv",
    "http://www.wikidata.org/prop/statement/value-normalized/": "psn",
    "http://www.wikidata.org/prop/qualifier/": "pq",
    "http://www.wikidata.org/prop/qualifier/value/": "pqv",
    "http://www.wikidata.org/prop/qualifier/value-normalized/": "pqn",
    "http://www.wikidata.org/prop/reference/": "pr",
    "http://www.wikidata.org/prop/reference/value/": "prv",
    "http://www.wikidata.org/prop/reference/value-normalized/": "prn",
    "http://www.wikidata.org/prop/novalue/": "wdno"
}



################ PARAM NAMES AND DOC

TARGET_CLASSES_PARAM = "target_classes"
"""
List of strings: List of target classes to associate a shape with
"""

TARGET_GRAPH_PARAM = "raw_graph"
"""
String: RDF content to be analyzed
"""

INPUT_FORMAT_PARAM = "input_format"
"""
String: RDF syntax used. Ntriples is used by default Accepted values -->

"nt" (n-triples)
"turtle" (turtle)
"xml" (RDF/XML)
"n3" (n3)
"json-ld" (JSON LD)
"tsv_spo" (lines with subject predicate and object separated by tab '\\t' chars 
"""


INSTANTIATION_PROPERTY_PARAM = "instantiation_prop"
"""
String: property used to links an instance with its class. rdf:type by default.
"""


NAMESPACES_TO_IGNORE_PARAM = "ignore"
"""
List of Strings: List of namespaces whose properties should be ignored during the shexing process.
"""

INFER_NUMERIC_TYPES_PARAM = "infer_untyped_nums"
"""
Bool: default, True. If True, it tries to infer the numeric type (xsd:int, xsd:float..) of 
untyped numeric literals 
"""

DISCARD_USELESS_CONSTRAINTS_PARAM = "discard_useless_constraints"
"""
Bool: default, True. default, True. If True, it keeps just the most possible specific constraint w.r.t. cardinality 
"""

ALL_INSTANCES_COMPLIANT_PARAM = "all_compliant"
"""
Bool: default, True. default, True. If False, the shapes produced may not be compliant with all the entities considered
to build them. This is because it won't use Kleene closeres for any constraint. 
"""

KEEP_LESS_SPECIFIC_PARAM = "keep_less_specific"
"""
Bool: default, True. It prefers to use "+" closures rather than exact cardinalities in the triple constraints
"""

ACEPTANCE_THRESHOLD_PARAM = "threshold"
"""
Float: number in [0,1] that indicates the minimum proportion of entities that should have a given feature for this
to be accepted as a triple constraint in the produced shape.
"""

ALL_CLASSES_MODE_PARAM = "all_classes"
"""
Bool: default, False. If True, it generates a shape for every elements with at least an instance 
in the considered graph.
"""
SHAPE_MAP_PARAM = "shape_map"
"""
String: shape map to associate nodes with shapes. It uses the same syntax of validation shape maps. 
"""

REMOTE_GRAPH_PARAM = "graph_url"
"""
String: URL to retrieve an online raw graph.
"""

ENDPOINT_GRAPH_PARAM = "endpoint"
"""
String: URL of an SPARQL endpoint.
"""

NAMESPACES_PARAM = "prefixes"
"""
Dict. key are namespaces and values are prefixes. The pairs key value provided here will be used 
to parse the RDF content and t write the resulting shapes.
"""

QUERY_DEPTH_PARAM = "query_depth"
"""
Integer: default, 1. It indicates the depth to generate queries when targeting a SPARQL endpoint.
Currently it can be 1 or 2.
"""

DISABLE_COMMENTS_PARAM = "disable_comments"
"""
Bool: default, False. When set to True, the shapes do not include comment 
with ratio of entities compliant with a triple constraint
"""

QUALIFIER_NAMESPACES_PARAM = "namespaces_for_qualifiers"
"""
List. Default, None. When a list with elements is provided, the properties in the namespaces specified are considered
to be pointers to qualifier nodes.
"""

SHAPE_QUALIFIERS_MODE_PARAM = "shape_qualifiers_mode"
"""
Bool: default, False. When set to true, a shape is generated for those nodes detected as qualifiers according to
Wikidata data model and the properties pointing to them specified in QUALIFIER_NAMESPACES_PARAM
"""



############ Param definition for a use case

shape_map = "SPARQL'SELECT DISTINCT ?virus WHERE {   VALUES ?virus {  wd:Q82069695  }  }'@<Virus>  " # wd:Q16983360 wd:Q16991954 wd:Q8351095  wd:Q16983356 wd:Q4902157  wd:Q278567


params = {NAMESPACES_PARAM: new_namespaces,
          SHAPE_MAP_PARAM: shape_map,
          ENDPOINT_GRAPH_PARAM: "https://query.wikidata.org/sparql",
          ALL_CLASSES_MODE_PARAM: False,
          QUERY_DEPTH_PARAM: 2,
          ACEPTANCE_THRESHOLD_PARAM: 0,
          INSTANTIATION_PROPERTY_PARAM: "http://www.wikidata.org/prop/direct/P31",
          DISABLE_COMMENTS_PARAM: True,
          SHAPE_QUALIFIERS_MODE_PARAM: True,
          QUALIFIER_NAMESPACES_PARAM: ["http://www.wikidata.org/prop/"]

          }

############ Calling web service

r = requests.post(url=url_web_service, json=params)


data = r.json()
print(data["result"])
print("Done!")



from shexer.shaper import Shaper

namespaces_dict = {
    "http://www.w3.org/2000/01/rdf-schema#": "rdfs",
    "http://www.wikidata.org/prop/": "p",
    "http://www.wikidata.org/prop/direct/": "wdt",
    "http://www.wikidata.org/entity/": "wd",
    "http://www.w3.org/2001/XMLSchema#": "xsd",
    "http://www.w3.org/1999/02/22-rdf-syntax-ns#": "rdf",
    "http://www.w3.org/XML/1998/namespace": "xml",
    "http://wikiba.se/ontology#": "wikibase",
    "http://schema.org/": "schema",
    "http://www.w3.org/2004/02/skos/core#": "skos"
}


"""
# shape_map_raw --> sheXer focus on some target structures to associate shapes with. 
                    You can use these shape maps: single-column SPARQL queries, node selectors, focus nodes.
                    You can use several selectors writting them in differente lines.
                    At the moment, each selector should be in a single line.
                    
                    Instead of shape maps, you can specify class URIs  with a list in the param  target_classes.
                    With that, all the instances of a given class will be used to build its shape.
                    
                    
# namespaces to ignore --> I'm telling shexer to ignore properties which are direct child of the namespaces in the
                    specified list.
# url_endpoint -->  Configured to work against Wikidata endpoint. It can work with local files as well and different
                    RDF syntax (shexer is a general RDF tool, not a Wikidata-specific one).
# instantiation_property ---> The property to instantiate classes is used in special ways at several points.
                    When it is not specified, rdf:type is used by default.
# namespaces_dict   ---> they will be used both to interpret prefixes in the shape maps and target RDF graph and
                    in the shex output.
# disable_comments   ---> Set to True to avoid getting the ratio of each triple constraint in the output.

# shape_qualifiers_mode AND namespaces_for_qualifier_props  ---> if shape_qualifiers_mode is set to True, then 
                    shexer will generate shapes for qualifiers. Shexer will consider qualifiers (according 
                    to Wikidata model) those nodes pointed by a property in any of the namespaces specified 
                    in 'namespaces_for_qualifier_props'
                    
#  depth_for_building_subgraph  ---> When using shexer against an endpoint, the library retrieves content starting
                   in the target nodes (specified via shape map, list of target classes, etc). All the triples in which
                   a target node is subject will be tracked. Then, it will repeat recursively this process with the
                   objects of those triples for a number of iterations specified in 'depth_for_building_subgraph'.
                   Depending on the targeted graph, setting this param to a high number could cause a huge performance 
                   cost. To get a shape for qualifiers, of the target nodes, the depth should be set to 2, at least.                
 
rest of params ---> shexer has multiple configuration params, but maybe just let those booleans as they are there 
                    for a first usage.                     

"""

shape_map_raw = "SPARQL'SELECT DISTINCT ?virus WHERE {   VALUES ?virus {  wd:Q82069695  }  }'@<Virus>  " # wd:Q8351095  wd:Q16983356 wd:Q4902157  wd:Q278567 wd:Q16983360 wd:Q16991954

shaper = Shaper(shape_map_raw=shape_map_raw,
                url_endpoint="https://query.wikidata.org/sparql",
                all_instances_are_compliant_mode=True,
                infer_numeric_types_for_untyped_literals=True,
                discard_useless_constraints_with_positive_closure=True,
                keep_less_specific=True,
                instantiation_property="http://www.wikidata.org/prop/direct/P31",
                namespaces_dict=namespaces_dict,
                depth_for_building_subgraph=2,
                namespaces_for_qualifier_props=["http://www.wikidata.org/prop/"],
                shape_qualifiers_mode=True,
                # disable_comments=True,
                all_classes_mode=False)




"""

string_output --> if set to true, result will be printed in the console. Alternatively you can specify an output
               file with a param output_file.
aceptance_threshold --> (I'm aware of the typo) it sets a threshold to ignore topological features associated to a
                  shape that happen in less than aceptance_threshold*100 percent of the entities used to build a shape.
                  

"""
result = shaper.shex_graph(string_output=True, acceptance_threshold=0)


print(result)
print("Done!")


from shexer.core.class_shexer import ClassShexer


def get_class_shexer(class_instances_target_dict,
                     class_profile_dict,
                     remove_empty_shapes,
                     original_target_classes,
                     original_shape_map,
                     discard_useless_constraints_with_positive_closure,
                     keep_less_specific,
                     all_compliant_mode,
                     instantiation_property,
                     disable_or_statements,
                     disable_comments,
                     namespaces_dict,
                     allow_opt_cardinality,
                     disable_exact_cardinality,
                     shapes_namespace):
    class_count_dicts = {}

    for a_class_key in class_instances_target_dict:
        class_count_dicts[a_class_key] = len(class_instances_target_dict[a_class_key])

    return ClassShexer(
        class_counts_dict=class_count_dicts,
        class_profile_dict=class_profile_dict,
        class_profile_json_file=None,
        remove_empty_shapes=remove_empty_shapes,
        original_target_classes=original_target_classes,
        original_shape_map=original_shape_map,
        discard_useless_constraints_with_positive_closure=discard_useless_constraints_with_positive_closure,
        keep_less_specific=keep_less_specific,
        all_compliant_mode=all_compliant_mode,
        instantiation_property=instantiation_property,
        disable_or_statements=disable_or_statements,
        disable_comments=disable_comments,
        namespaces_dict=namespaces_dict,
        allow_opt_cardinality=allow_opt_cardinality,
        disable_exact_cardinality=disable_exact_cardinality,
        shapes_namespace=shapes_namespace,
    )



class AbstractInstanceTracker(object):

    def track_instances(self):
        raise NotImplementedError()


    @property
    def disambiguator_prefix(self):
        """
        It return an str that may help for disambiguation purposes if the instance_tracker is used to produce dicts
        that may be integrated with other instance dicts and there should be any key colission.
        :return:
        """
        raise NotImplementedError()

    @property
    def disambiguator_prefix(self):
        return "mixed_"


from shexer.shaper import Shaper
from shexer.consts import NT, SHEXC, SHACL_TURTLE

target_graph = """
<http://example.org/Jimmy> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://xmlns.com/foaf/0.1/Person> .
<http://example.org/Jimmy> <http://xmlns.com/foaf/0.1/age> "23"^^<http://www.w3.org/2001/XMLSchema#integer> .
<http://example.org/Jimmy> <http://xmlns.com/foaf/0.1/name> "Jimmy" .
<http://example.org/Jimmy> <http://xmlns.com/foaf/0.1/familyName> "Jones" .
<http://example.org/Sarah> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://xmlns.com/foaf/0.1/Person> .
<http://example.org/Sarah> <http://xmlns.com/foaf/0.1/age> "22"^^<http://www.w3.org/2001/XMLSchema#integer> .
<http://example.org/Sarah> <http://xmlns.com/foaf/0.1/name> "Sarah" .
<http://example.org/Sarah> <http://xmlns.com/foaf/0.1/familyName> "Salem" .
<http://example.org/Bella> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://xmlns.com/foaf/0.1/Person> .
<http://example.org/Bella> <http://xmlns.com/foaf/0.1/age> "56"^^<http://www.w3.org/2001/XMLSchema#integer> .
<http://example.org/Bella> <http://xmlns.com/foaf/0.1/name> "Isabella" .
<http://example.org/David> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://xmlns.com/foaf/0.1/Person> .
<http://example.org/David> <http://xmlns.com/foaf/0.1/name> "David" .
<http://example.org/David> <http://xmlns.com/foaf/0.1/familyName> "Doulofeau" .
<http://example.org/David> <http://xmlns.com/foaf/0.1/knows> <http://example.org/Sarah> .
<http://example.org/HumanLike> <http://xmlns.com/foaf/0.1/name> "Person" .
<http://example.org/HumanLike> <http://xmlns.com/foaf/0.1/familyName> "Maybe" .
<http://example.org/HumanLike> <http://xmlns.com/foaf/0.1/age> "99"^^<http://www.w3.org/2001/XMLSchema#integer> .
<http://example.org/HumanLike> <http://xmlns.com/foaf/0.1/knows> <http://example.org/David> .
<http://example.org/x1> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://xmlns.com/foaf/0.1/Document> .
<http://example.org/x1> <http://xmlns.com/foaf/0.1/depiction> "A thing that s nice" .
<http://example.org/x1> <http://xmlns.com/foaf/0.1/title> "A nice thing" .
<http://example.org/x2> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://xmlns.com/foaf/0.1/Document> .
<http://example.org/x2> <http://xmlns.com/foaf/0.1/title> "Another thing" .
"""


namespaces = {"http://example.org/": "ex",
            "http://www.w3.org/XML/1998/namespace/": "xml",
            "http://www.w3.org/1999/02/22-rdf-syntax-ns#": "rdf",
            "http://www.w3.org/2000/01/rdf-schema#": "rdfs",
            "http://www.w3.org/2001/XMLSchema#": "xsd",
            "http://xmlns.com/foaf/0.1/": "foaf"
            }

shaper = Shaper(raw_graph=target_graph,
                namespaces_dict=namespaces,
                all_classes_mode=True,
                allow_opt_cardinality=False)

res = shaper.shex_graph(string_output=True,
                        output_format=SHEXC,
                        acceptance_threshold=0.4)
print(res)


from shexer.core.instances.annotators.strategy_mode.base_strategy_mode import BaseStrategyMode
from shexer.core.instances.pconsts import _S, _P, _O

class AllClasesMode(BaseStrategyMode):

    def __init__(self, annotator_ref):
        super().__init__(annotator_ref)



    def is_relevant_triple(self, a_triple):
        if a_triple[_P] != self._instantiation_property:
            return False
        return True


    def annotate_triple(self, a_triple):
        if self._instance_tracker.is_an_instantiation_prop(a_triple[_P]):
            self._annotator_ref.add_new_class_to_instances_dict(a_triple[_O].iri)
            self._annotator_ref.annotate_instance(a_triple)





import unittest
from shexer.shaper import Shaper
from test.const import G1, BASE_FILES, default_namespaces, G1_ALL_CLASSES_NO_COMMENTS
from test.t_utils import file_vs_str_tunned_comparison

from shexer.consts import TURTLE

_BASE_DIR = BASE_FILES + "discard_and_compliant\\"

class TestDiscardUselessConstraintsAndCompliantMode(unittest.TestCase):

    def test_no_compilant_no_discard(self):
        shaper = Shaper(target_classes=["http://xmlns.com/foaf/0.1/Person",
                                        "http://xmlns.com/foaf/0.1/Document"],
                        graph_file_input=G1,
                        namespaces_dict=default_namespaces(),
                        all_classes_mode=False,
                        input_format=TURTLE,
                        disable_comments=True,
                        discard_useless_constraints_with_positive_closure=False,
                        all_instances_are_compliant_mode=False)
        str_result = shaper.shex_graph(string_output=True)
        self.assertTrue(file_vs_str_tunned_comparison(file_path=_BASE_DIR + "no_compliant_no_discard.shex",
                                                      str_target=str_result))

    def test_compilant_no_discard(self):
        shaper = Shaper(target_classes=["http://xmlns.com/foaf/0.1/Person",
                                        "http://xmlns.com/foaf/0.1/Document"],
                        graph_file_input=G1,
                        namespaces_dict=default_namespaces(),
                        all_classes_mode=False,
                        input_format=TURTLE,
                        disable_comments=True,
                        discard_useless_constraints_with_positive_closure=False,
                        all_instances_are_compliant_mode=True)
        str_result = shaper.shex_graph(string_output=True)
        self.assertTrue(file_vs_str_tunned_comparison(file_path=_BASE_DIR + "compliant_no_discard.shex",
                                                      str_target=str_result))

    def test_no_compilant_discard(self):
        shaper = Shaper(target_classes=["http://xmlns.com/foaf/0.1/Person",
                                        "http://xmlns.com/foaf/0.1/Document"],
                        graph_file_input=G1,
                        namespaces_dict=default_namespaces(),
                        all_classes_mode=False,
                        input_format=TURTLE,
                        disable_comments=True,
                        discard_useless_constraints_with_positive_closure=True,
                        all_instances_are_compliant_mode=False)
        str_result = shaper.shex_graph(string_output=True)
        self.assertTrue(file_vs_str_tunned_comparison(file_path=_BASE_DIR + "no_compliant_discard.shex",
                                                      str_target=str_result))

    def test_compilant_discard(self):
        shaper = Shaper(target_classes=["http://xmlns.com/foaf/0.1/Person",
                                        "http://xmlns.com/foaf/0.1/Document"],
                        graph_file_input=G1,
                        namespaces_dict=default_namespaces(),
                        all_classes_mode=False,
                        input_format=TURTLE,
                        disable_comments=True,
                        discard_useless_constraints_with_positive_closure=True,  # default value
                        all_instances_are_compliant_mode=True)  # default value
        str_result = shaper.shex_graph(string_output=True)
        self.assertTrue(file_vs_str_tunned_comparison(file_path=G1_ALL_CLASSES_NO_COMMENTS,
                                                      str_target=str_result))


import unittest
from shexer.shaper import Shaper
from test.const import G1, BASE_FILES, default_namespaces, G1_ALL_CLASSES_NO_COMMENTS
from test.t_utils import file_vs_str_tunned_comparison

from shexer.consts import TURTLE

_BASE_DIR = BASE_FILES + "instantiation_prop\\"  # We just need something with another instantiation property


class TestAllClasesMode(unittest.TestCase):

    def test_all_classes_g1(self):
        shaper = Shaper(
            graph_file_input=G1,
            namespaces_dict=default_namespaces(),
            all_classes_mode=True,
            input_format=TURTLE,
            disable_comments=True)
        str_result = shaper.shex_graph(string_output=True)
        self.assertTrue(file_vs_str_tunned_comparison(file_path=G1_ALL_CLASSES_NO_COMMENTS,
                                                      str_target=str_result))

    def test_all_classes_ex_a(self):
        shaper = Shaper(
            graph_file_input=_BASE_DIR + "G1_ex_a.ttl",
            namespaces_dict=default_namespaces(),
            all_classes_mode=True,
            input_format=TURTLE,
            instantiation_property="http://example.org/a",
            disable_comments=True)
        str_result = shaper.shex_graph(string_output=True)
        self.assertTrue(file_vs_str_tunned_comparison(file_path=_BASE_DIR + "G1_ex_a.shex",
                                                      str_target=str_result))

